/* query_mach_debug.c - TEST CODE
 
   Copyright (C) 1998,1999,2001,02,2006 Free Software Foundation, Inc.
   Gordon Matzigkeit <gord@fig.org>, 1999
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

*/

#define _GNU_SOURCE 1

#include <hurd.h>
#include <hurd/io.h>

#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <error.h>

//#include <argp.h>

#include "mach_debug/mach_debug.h"


/* Test Functions */
void host_zone_info_test(host_t h)
{
  zone_name_array_t names;
  zone_info_array_t info;
  mach_msg_type_number_t namesCnt;
  mach_msg_type_number_t infoCnt;

  printf("\nTest: host_zone_info\n");

  host_zone_info(h, &names, &namesCnt, &info, &infoCnt); /* Call the mach_debug interface */


//////VERY DUMMY---JUST FOR TEST THE TEST
  printf("\tRESULTS FOUND: %d\n",infoCnt);
  if(infoCnt <= 0)
    return;
  printf("\tZone name 0: %s \n",names[0].zn_name);
  printf("\tZone info 0: %li \n",info[0].zi_cur_size);

}

void host_stack_usage_test(host_t h)
{
  vm_size_t reserved, space, resident, maxusage;
  unsigned total;
  vm_offset_t maxstack;
  kern_return_t ret;

  printf("\nTest: host_stack_usage\n");

  ret = host_stack_usage(h, &reserved, &total, &space, &resident, &maxusage, &maxstack); /* Call the mach_debug interface */

  if(ret == KERN_INVALID_HOST)
    printf("\tERROR: Invalid host");
  else if (ret == KERN_RESOURCE_SHORTAGE)
    printf("\tERROR: Couldn't allocate memory");
  else
    printf("\treserved=%d\n\ttotal=%d\n\tspace=%d\n\tresident=%d\n\tmazusage=%d\n\tmaxstack=%d\n", reserved, total, space, resident, maxusage, maxstack);
}

void host_virtual_physical_table_info_test(host_t h)
{
  hash_info_bucket_array_t info; 
  mach_msg_type_number_t infoCnt;
  int i=0;
  kern_return_t ret;

  printf("\nTest: host_virtual_physical_table_info\n");

  ret = host_virtual_physical_table_info(h, &info, &infoCnt); /* Call the mach_debug interface */

  if(ret == KERN_INVALID_HOST)
    printf("\tERROR: Invalid host");
  else if (ret == KERN_RESOURCE_SHORTAGE)
    printf("\tERROR: Couldn't allocate memory");
  else{
    //BUGGY , DUMMY --- HARD TEST TEST: this read just the first result.
    printf("\tRESULTS FOUND: %d\n",infoCnt);
    if(infoCnt <= 0)
      return;
    printf("\thib_count=%d\n",info[0].hib_count);
  }
}

/* End test functions */


/* * * * * * * * * * * *		
 * 	M A I N 	*
 * * * * * * * * * * * */
int
main (int argc, char **argv)
{
  error_t err;
  mach_port_t bootstrap;
  host_t host = (host_t) bootstrap;

  task_get_bootstrap_port (mach_task_self (), &bootstrap);

  printf("\n\t*****\tStart test program for mach_debug interface\t*****\n\n");

  /* Begin Tests ... */

  // host_zone_info_test(host); //DON'T WORK, why?
  host_stack_usage_test(host);
  host_virtual_physical_table_info_test(host);

  /* ...End Tests */

  mach_port_deallocate (mach_task_self (), bootstrap);

  printf("\n\t*****\t End test program for mach_debug interface \t*****\n\n");

  return 0;
}
